/*========================== begin_copyright_notice ============================

Copyright (C) 2023-2024 Intel Corporation

SPDX-License-Identifier: MIT

============================= end_copyright_notice ===========================*/

#include "GenX.h"

#include <unordered_map>
#include <unordered_set>

static inline constexpr uint32_t encodeGmdId(uint32_t Major, uint32_t Minor,
                                             uint32_t Revision) {
  return ((Major & 0x3ff) << 22 | (Minor & 0xff) << 14 | (Revision & 0x3f));
}

struct TargetProperties {
  bool HasFP64;
  bool HasBFloat16;
  bool HasSLMCasInt64;
  bool HasDpas;
  bool HasDpasw;
  bool HasDpasFp16;
  bool HasDpasBf16;
  bool HasDpasTf32;

  unsigned GrfWidth;
  std::unordered_set<unsigned> SupportedGrfNums;

  unsigned MaxSLMSize;
};

// clang-format off
static const std::unordered_map<uint32_t, TargetProperties> TargetProps = {
  { encodeGmdId(12, 71, 4),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 71, 0),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 70, 4),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 70, 0),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 61, 7),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ true,
      /*.HasSLMCasInt64 =*/ true,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 512,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 128, }, },
  { encodeGmdId(12, 60, 7),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ true,
      /*.HasSLMCasInt64 =*/ true,
      /*.HasDpas =*/ true,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ true,
      /*.HasDpasBf16 =*/ true,
      /*.HasDpasTf32 =*/ true,
      /*.GrfWidth =*/ 512,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 128, }, },
  { encodeGmdId(12, 60, 6),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ true,
      /*.HasSLMCasInt64 =*/ true,
      /*.HasDpas =*/ true,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ true,
      /*.HasDpasBf16 =*/ true,
      /*.HasDpasTf32 =*/ true,
      /*.GrfWidth =*/ 512,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 128, }, },
  { encodeGmdId(12, 60, 5),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ true,
      /*.HasSLMCasInt64 =*/ true,
      /*.HasDpas =*/ true,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ true,
      /*.HasDpasBf16 =*/ true,
      /*.HasDpasTf32 =*/ true,
      /*.GrfWidth =*/ 512,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 128, }, },
  { encodeGmdId(12, 60, 3),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ true,
      /*.HasSLMCasInt64 =*/ true,
      /*.HasDpas =*/ true,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ true,
      /*.HasDpasBf16 =*/ true,
      /*.HasDpasTf32 =*/ true,
      /*.GrfWidth =*/ 512,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 128, }, },
  { encodeGmdId(12, 60, 1),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ true,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ true,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ true,
      /*.HasDpasBf16 =*/ true,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 512,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 128, }, },
  { encodeGmdId(12, 60, 0),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ true,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ true,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ true,
      /*.HasDpasBf16 =*/ true,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 512,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 128, }, },
  { encodeGmdId(12, 57, 0),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ true,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ true,
      /*.HasDpasw =*/ true,
      /*.HasDpasFp16 =*/ true,
      /*.HasDpasBf16 =*/ true,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 56, 5),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ true,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ true,
      /*.HasDpasw =*/ true,
      /*.HasDpasFp16 =*/ true,
      /*.HasDpasBf16 =*/ true,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 56, 4),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ true,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ true,
      /*.HasDpasw =*/ true,
      /*.HasDpasFp16 =*/ true,
      /*.HasDpasBf16 =*/ true,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 56, 0),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ true,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ true,
      /*.HasDpasw =*/ true,
      /*.HasDpasFp16 =*/ true,
      /*.HasDpasBf16 =*/ true,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 55, 8),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ true,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ true,
      /*.HasDpasw =*/ true,
      /*.HasDpasFp16 =*/ true,
      /*.HasDpasBf16 =*/ true,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 55, 4),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ true,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ true,
      /*.HasDpasw =*/ true,
      /*.HasDpasFp16 =*/ true,
      /*.HasDpasBf16 =*/ true,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 55, 1),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ true,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ true,
      /*.HasDpasw =*/ true,
      /*.HasDpasFp16 =*/ true,
      /*.HasDpasBf16 =*/ true,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 55, 0),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ true,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ true,
      /*.HasDpasw =*/ true,
      /*.HasDpasFp16 =*/ true,
      /*.HasDpasBf16 =*/ true,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 50, 4),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ true,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ true,
      /*.HasDpasw =*/ true,
      /*.HasDpasFp16 =*/ true,
      /*.HasDpasBf16 =*/ true,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128,256},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 10, 0),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 4, 0),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 3, 0),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 2, 0),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 2, 0),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 1, 0),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(12, 0, 0),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(11, 2, 0),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(11, 0, 0),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(9, 7, 0),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(9, 6, 0),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(9, 5, 0),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(9, 4, 0),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(9, 3, 0),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(9, 2, 9),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(9, 1, 9),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(9, 0, 9),
    { /*.HasFP64 =*/ true,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
  { encodeGmdId(8, 0, 0),
    { /*.HasFP64 =*/ false,
      /*.HasBFloat16 =*/ false,
      /*.HasSLMCasInt64 =*/ false,
      /*.HasDpas =*/ false,
      /*.HasDpasw =*/ false,
      /*.HasDpasFp16 =*/ false,
      /*.HasDpasBf16 =*/ false,
      /*.HasDpasTf32 =*/ false,
      /*.GrfWidth =*/ 256,
      /*.SupportedGrfNums =*/ {128},
      /*.MaxSLMSize =*/ 64, }, },
};
// clang-format on

void clang::targets::GenXTargetInfo::setCPUProperties() {
  auto CPUId = encodeGmdId(Major, Minor, Revision);
  auto It = TargetProps.find(CPUId);
  if (It == std::end(TargetProps))
    return;

  const auto &Prop = It->second;
  HasFP64 = Prop.HasFP64;
  HasBFloat16 = Prop.HasBFloat16;
  HasSLMCasInt64 = Prop.HasSLMCasInt64;

  HasDpas = Prop.HasDpas;
  HasDpasw = Prop.HasDpasw;
  HasDpasFp16 = Prop.HasDpasFp16;
  HasDpasBf16 = Prop.HasDpasBf16;
  HasDpasTf32 = Prop.HasDpasTf32;

  GrfWidth = Prop.GrfWidth;
  SupportedGrfNums = Prop.SupportedGrfNums;

  MaxSLMSize = Prop.MaxSLMSize;
}
